#region Copyright Syncfusion Inc. 2001 - 2005
//
//  Copyright Syncfusion Inc. 2001 - 2005. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Collections;
using System.ComponentModel;

namespace Advent.MxRules.Win.ExternalClient
{

	/// <summary>
	///		A strongly-typed read-only collection of <see cref="UserAction"/> objects using 
	///		ITypedList implementation and a custom "Description" PropertyDescriptor 
	///		that returns state in format "NC - North Carolina"
	/// </summary>
	[Serializable]
	public class UserActionCollection : ArrayList, ITypedList
	{
		public new UserAction this[int index]
		{
			get
			{
				return (UserAction) base[index];
			}
			set
			{
				base[index] = value;
			}
		}

		public static UserActionCollection CreateDefaultCollection()
		{
			UserActionCollection states = new UserActionCollection();
            //states.Add(new UserAction("1", "Define Rule"));
            //states.Add(new UserAction("2", "Assign Rule"));
            states.Add(new UserAction("3", "Override"));
            states.Add(new UserAction("4", "Request Override"));
            //states.Add(new UserAction("5", "Approve Override"));
            //states.Add(new UserAction("6", "Deny Override"));
            states.Add(new UserAction("7", "Add Comment"));
            states.Add(new UserAction("8", "Exclude Portfolio"));
            //states.Add(new UserAction("9", "Resolve Exception"));

			return states;
		}

		public override bool IsReadOnly
		{
			get
			{
				// when both IsReadOnly and IsFixedSize return true the collection 
				// is not editable and no pencil will show up in foreign key dropdown list
				return true;   
			}
		}

		public override bool IsFixedSize
		{
			get
			{
				return true;
			}
		}

		#region ITypedList Members

		public PropertyDescriptorCollection GetItemProperties(PropertyDescriptor[] listAccessors)
		{
			ArrayList al = new ArrayList();
			PropertyDescriptorCollection pdc = TypeDescriptor.GetProperties(typeof(UserAction));
			al.AddRange(pdc);
			al.Add(new UserActionPropertyDescriptor("Description"));

			return new PropertyDescriptorCollection(((PropertyDescriptor[]) al.ToArray(typeof(PropertyDescriptor))));
		}

		public string GetListName(PropertyDescriptor[] listAccessors)
		{
			return "USStates";
		}

		#endregion
	}


	[Serializable]
	public class UserAction
	{
		private string _code;
		private string _name;

		public  UserAction()
		{
		}

		public  UserAction(string key, string name)
		{
			this._code = key;
			this._name = name;
		}

		[Browsable(true)]
		public string Key
		{
			get
			{
				return _code;
			}
			set
			{
				_code = value;
			}
		}

		[Browsable(true)]
		public string Name
		{
      
			get
			{
				return _name ;
			}
			set
			{
				_name = value;
			}
		}

		public override string ToString()
		{
			return this._name + "(" + this._code + ")";
		}
	}


	/// <summary>
	/// Readonly property descriptor. Returns state in format "NC - North Carolina"
	/// </summary>
	public class UserActionPropertyDescriptor : PropertyDescriptor
	{
		public UserActionPropertyDescriptor(string name)
			: base(name, null)
		{
		}

		/// <override/>
		public override bool ShouldSerializeValue(object component)
		{
			return false;
		}


		/// <override/>
		public override void SetValue(object component, object value)
		{
		}


		/// <override/>
		public override void ResetValue(object component)
		{
		}


		/// <override/>
		public override object GetValue(object component)
		{
			UserAction state = (UserAction) component;
            return String.Format(state.Key + " - " + state.Name);
		}


		/// <override/>
		public override bool CanResetValue(object component)
		{
			return false;
		}


		/// <override/>
		public override Type PropertyType
		{
			get
			{
				return typeof(string);
			}
		}

		/// <override/>
		public override bool IsReadOnly
		{
			get
			{
				return true;
			}
		}

		/// <override/>
		public override Type ComponentType
		{
			get
			{
				return typeof(UserAction);
			}
		}

		/// <override/>
		public override bool IsBrowsable
		{
			get
			{
				return true;
			}
		}

		/// <override/>
		public override int GetHashCode()
		{
			return base.GetHashCode();
		}


		/// <override/>
		public override bool Equals(object other)
		{
			if ((other is UserActionPropertyDescriptor))
			{
				return ((UserActionPropertyDescriptor) other).Name == this.Name;
			}
			return false;
		}

	}
}
